!  Program Name:
!  Author(s)/Contact(s):
!  Abstract:
!  History Log:
! 
!  Usage:
!  Parameters: <Specify typical arguments passed>
!  Input Files:
!        <list file names and briefly describe the data they include>
!  Output Files:
!        <list file names and briefly describe the information they include>
! 
!  Condition codes:
!        <list exit condition or error codes returned >
!        If appropriate, descriptive troubleshooting instructions or
!        likely causes for failures could be mentioned here with the
!        appropriate error code
! 
!  User controllable options: <if applicable>

Module module_namelist

#ifdef MPP_LAND
          USE module_mpp_land
#endif

    IMPLICIT NONE
    INTEGER, PARAMETER :: max_domain=5

#include "namelist.inc"
    TYPE(namelist_rt_field) , dimension(max_domain) :: nlst_rt
    save nlst_rt 

CONTAINS 

    subroutine read_rt_nlst(nlst)     
          implicit none

          TYPE(namelist_rt_field) nlst

          integer ierr, domain_msk_opt
          integer:: RT_OPTION, CHANRTSWCRT, channel_option, &
                    SUBRTSWCRT,OVRTSWCRT,AGGFACTRT, &
                    GWBASESWCRT,  GW_RESTART,RSTRT_SWC,TERADJ_SOLAR, &
                    sys_cpl, rst_typ, rst_bi_in, rst_bi_out, &
                    gwChanCondSw, GwPreCycles, GwSpinCycles, GwPreDiagInterval, gwsoilcpl, &
                    UDMP_OPT, overbank, bwOption, river2land
          real :: overFlowRate
          real ::  bankWght, minSlope, maxSlope
          real:: DTRT_TER,DTRT_CH,dxrt, gwChanCondConstIn, gwChanCondConstOut, gwIhShift
          logical :: fexist
          character(len=256) :: route_topo_f=""
          character(len=256) :: route_chan_f=""
          character(len=256) :: route_link_f=""
          character(len=256) :: route_lake_f=""
          character(len=256) :: route_direction_f=""
          character(len=256) :: route_order_f=""
          character(len=256) :: gwbasmskfil =""
          character(len=256) :: gwstrmfil =""
          character(len=256) :: geo_finegrid_flnm =""
          character(len=256) :: udmap_file =""
          character(len=256) :: GWBUCKPARM_file = ""
          character(len=256) :: streamGeoFile   = ""
       integer :: SOLVEG_INITSWC
       real out_dt, rst_dt
       character(len=256)  :: RESTART_FILE = ""
       logical            :: GwPreDiag, GwSpinUp
       integer            :: split_output_count, order_to_write
       integer :: igrid, iocflag
       character(len=256) :: geo_static_flnm = ""
       integer  :: DEEPGWSPIN

       integer :: i
       real :: dx1, dx2

       
          integer ::CHRTOUT_DOMAIN           ! Netcdf point timeseries output at all channel points
          integer ::CHRTOUT_GRID                ! Netcdf grid of channel streamflow values
          integer ::LSMOUT_DOMAIN              ! Netcdf grid of variables passed between LSM and routing components
          integer ::RTOUT_DOMAIN                ! Netcdf grid of terrain routing variables on routing grid
          integer  :: output_gw
          integer  :: outlake
          integer  :: outStatic, unstableFlag,urbMinOrd
          real :: trapCoef

        
!!!!!  add for dhydro coupling
       integer :: dhydroCplOpt !!! 0, no; 1: fully feedback of surface head; 2: partial feed back of surface head
       real :: cplTime     !!! time interval for coupling feed back. in seconds
       real :: RETDEPRT0



!!! add the following two dummy variables 
       integer  :: NSOIL
       real :: ZSOIL8(8)

       logical            :: dir_e
#ifdef WRF_HYDRO_NUDGING
       character(len=256) :: nudgingParamFile
       character(len=256) :: netwkReExFile
       logical            :: readTimesliceParallel
       logical            :: temporalPersistence
       character(len=256) :: nudgingLastObsFile
#endif 
       character(len=256) :: segInfoFile
       integer :: dabe
       real :: factorIn, minDTCT0, baseflowFactor


       integer ::checkLag     !! 1 for on, 0 for off.A
       integer ::checkRiver   !! 1 for on,             
       character(len=19) :: waitDate   !!! give a date (yyyy_mm_dd_hh:mm:ss)
                                       ! when reach the time, check the flag file)
       integer :: waitMaxTime  !!! seconds for maximum time
       integer :: sleepSeconds !!! set default as 5 seconds
       character(len=256) :: lagFlagFile  !!! lagFlagFile used to terminate the waiting time.


       namelist /HYDRO_nlist/ NSOIL, ZSOIL8,&
            checkRiver,&
            RESTART_FILE,SPLIT_OUTPUT_COUNT,IGRID,&
            geo_static_flnm, &
            out_dt, rst_dt, &
            DEEPGWSPIN, SOLVEG_INITSWC, &
            RT_OPTION, CHANRTSWCRT, channel_option, &
                    SUBRTSWCRT,OVRTSWCRT,AGGFACTRT, dtrt_ter,dtrt_ch,dxrt,&
                    GwSpinCycles, GwPreCycles, GwSpinUp, GwPreDiag, GwPreDiagInterval, gwIhShift, &
                    GWBASESWCRT, gwChanCondSw, gwChanCondConstIn, gwChanCondConstOut , &
                    route_topo_f,route_chan_f,route_link_f,route_lake_f, &
                    route_direction_f,route_order_f,gwbasmskfil, geo_finegrid_flnm,&
                    gwstrmfil,GW_RESTART,RSTRT_SWC,TERADJ_SOLAR, sys_cpl, &
                    order_to_write , rst_typ, rst_bi_in, rst_bi_out, gwsoilcpl, &
                    CHRTOUT_DOMAIN,CHRTOUT_GRID,LSMOUT_DOMAIN,RTOUT_DOMAIN, output_gw, outlake, udmap_file, &
                    UDMP_OPT, GWBUCKPARM_file, iocflag  , outStatic,  unstableFlag,trapCoef,domain_msk_opt, dabe, &
                    dhydroCplOpt, cplTime, factorIn, baseflowFactor,minSlope, maxSlope, minDTCT0, RETDEPRT0, &
                    segInfoFile,urbMinOrd

        namelist /LAGCHECK_nlst/ checkLag, waitDate, waitMaxTime,sleepSeconds, lagFlagFile

        namelist /lndRiver_nlst/ overbank, bankWght, bwOption, streamGeoFile, river2land, overFlowRate


#ifdef WRF_HYDRO_NUDGING
   namelist /NUDGING_nlist/ nudgingParamFile,      netwkReExFile,       &
                            readTimesliceParallel, temporalPersistence, &
                            nudgingLastObsFile
   ! Default values... 
   nudgingParamFile = "DOMAIN/nudgingParams.nc"
   netwkReExFile    = "DOMAIN/netwkReExFile.nc"
   readTimesliceParallel = .true.
   temporalPersistence   = .true.
   nudgingLastObsFile = ""
#endif 
   unstableFlag = 0 !!! 1: stop the code. 0: continue
   UDMP_OPT = 0
   rst_bi_in = 0
   rst_bi_out = 0
   iocflag = 0
        dhydroCplOpt = 0
        cplTime = 0
   overbank = 0

   checkRiver = 0
                    
#ifndef NCEP_WCOSS
          open(12, file="hydro.namelist", form="FORMATTED")
#else
          open(12, form="FORMATTED")
#endif
          read(12, HYDRO_nlist, iostat=ierr)
          if(ierr .ne. 0) call hydro_stop("HYDRO_nlst namelist error in HYDRO_nlist ")

          rewind(12)
          read(12, LAGCHECK_nlst, iostat=ierr)
          if(ierr .ne. 0) call hydro_stop("HYDRO_nlst namelist error in LAGCHECK_nlst")

          rewind(12)
          read(12, lndRiver_nlst, iostat=ierr)
          if(ierr .ne. 0) call hydro_stop("lndRiver_nlst namelist error in lndRiver_nlst")

#ifdef WRF_HYDRO_NUDGING          
          rewind(12)
          read(12, NUDGING_nlist, iostat=ierr)
          if(ierr .ne. 0) call hydro_stop("NUDGING_nlst namelist error in read_rt_nlst")
#endif

!          rewind(12)
!          read(12, HYDRO_DABE, iostat=ierr)
!          if(ierr .ne. 0) call hydro_stop("lndRiver_nlst namelist error in HYDRO_DABE")

           close(12)

           nlst%segInfoFile = segInfoFile



      nlst%urbMinOrd = urbMinOrd
      if(outStatic .eq. 1) overbank = 0
      inquire (file=trim(streamGeoFile), exist=fexist)
      if(.not. fexist .and. bwOption .ne. 0) then 
          write(6,*)  "Warning:  streamGeoFile does not exist. Rest bwOption to be 0 "
          bwOption = 0
      endif
      if(.not. fexist .and. overbank .eq. 2) then 
         call hydro_stop ("Error:  streamGeoFile does not exist. Please rest overbank from hydro.namelist. ")
      endif
      if(overbank .eq. 2 .and. bwOption .eq. 0) then 
         write(6,*)  "Warning: reset bwOption to be 1 ... "
      endif

      nlst%OVRTSWCRT0 = OVRTSWCRT 

      if(OVRTSWCRT .eq. 0) OVRTSWCRT = 1
      write(6,*) "OVRTSWCRT0 ", nlst%OVRTSWCRT0

      if(trapCoef .eq. 0 ) trapCoef = 0.05
      nlst%trapCoef     = trapCoef
      nlst%unstableFlag = unstableFlag
!!!! when stream geometry is available 
         nlst%overbank = overbank
         nlst%bankWght = bankWght
         nlst%streamGeoFile = streamGeoFile 
         nlst%bwOption      = bwOption 

         nlst%river2land    = river2land    
         nlst%overFlowRate  = overFlowRate  


      write(6,*) "nlst%NSOIL = ", nlst%NSOIL
      write(6,*) "nlst%ZSOIL8 = ",nlst%ZSOIL8

!  nlst%DT = DT
  nlst%RESTART_FILE = RESTART_FILE
  nlst%SPLIT_OUTPUT_COUNT = SPLIT_OUTPUT_COUNT
  nlst%IGRID = IGRID
  nlst%iocflag = iocflag
  nlst%geo_static_flnm = geo_static_flnm
  nlst%out_dt = out_dt
  nlst%rst_dt = rst_dt
  nlst%DEEPGWSPIN = DEEPGWSPIN
  nlst%SOLVEG_INITSWC = SOLVEG_INITSWC
  nlst%checkRiver = checkRiver


  write(nlst%hgrid,'(I1)') igrid

!!! dhydro coupling
   nlst%dhydroCplOpt = dhydroCplOpt
   nlst%cplTime = cplTime
   nlst%RETDEPRT0 = RETDEPRT0


  if(RESTART_FILE .eq. "") rst_typ = 0

  if(rst_bi_out .eq. 1) then
! This part works for intel not pgi
!     inquire(directory='restart', exist=dir_e)
      inquire(file='restart/.', exist=dir_e)
      if(.not. dir_e) then
         call system('mkdir restart')
      endif
  endif


  if(outStatic .eq. 1) then 
       write(6,*)  "outStatic is 1, force dabe to be zero ..... "
       dabe = 0
  endif


!!
    if(channel_option .eq. 5 .and. overbank .eq. 2) then
       call hydro_stop("Error: no overbank = 2 for channel_option = 5")
    endif
    if(channel_option .eq. 5) then
       !!! Muskingam-gridded
       channel_option = 3
       UDMP_OPT = -1
    endif
    
 
! run Rapid 
    if(channel_option .eq. 4) then
       write(6,*)  "channel_option is 4 , running rapid .... "
       write(6,*)  "set CHANRTSWCRT, OVRTSWCRT , SUBRTSWCRT all to be 0."
       CHANRTSWCRT = 0
       OVRTSWCRT = 0
       SUBRTSWCRT = 0
    endif
    if(nlst%dhydroFlag .eq. 1) then 
       write(6,*)  "called from namelist, using dhydro.exe interface. "
       write(6,*)  "set OVRTSWCRT  to be 1 ...... "
       if(OVRTSWCRT .eq. 0 .and. SUBRTSWCRT .eq. 0) OVRTSWCRT  = 1
    else
       if(sys_cpl .eq. 1) then
           write(6,*)  "called from namelist, using wrf_hydro.exe interface. "
       else
           write(6,*)  "called from namelist, using fully coupled interface. "
       endif
    endif
    if(OVRTSWCRT .eq. 0) then 
       write(6,*) "no overland routing, set RTOUT_DOMAIN=0"
       RTOUT_DOMAIN = 0
    endif

    nlst%outStatic      = outStatic
    nlst%CHRTOUT_DOMAIN = CHRTOUT_DOMAIN
    nlst%output_gw      = output_gw
    nlst%outlake      = outlake
    nlst%CHRTOUT_GRID = CHRTOUT_GRID
    nlst%LSMOUT_DOMAIN = LSMOUT_DOMAIN
    nlst%RTOUT_DOMAIN = RTOUT_DOMAIN
    nlst%RT_OPTION = RT_OPTION
    nlst%CHANRTSWCRT = CHANRTSWCRT
    nlst%GW_RESTART  = GW_RESTART 
    nlst%RSTRT_SWC   = RSTRT_SWC  
    nlst%channel_option = channel_option
    nlst%DTRT_TER   = DTRT_TER
    nlst%DTRT_CH   = DTRT_CH
    nlst%DTCT      = DTRT_CH   ! small time step for grid based channel routing
    write(6,*) "check namelist, RTOUT_DOMAIN, OVRTSWCRT ..", RTOUT_DOMAIN, OVRTSWCRT
#ifdef MPP_LAND
  if(my_id .eq. IO_id) then
#endif
    if(nlst%DT .lt. DTRT_CH) then 
          write(6,*)  "nlst%DT,  DTRT_CH = ",nlst%DT,  DTRT_CH
          write(6,*)  "reset DTRT_CH=nlst%DT "
          DTRT_CH=nlst%DT
    endif
    if(nlst%DT .lt. DTRT_TER) then 
          write(6,*)  "nlst%DT,  DTRT_TER = ",nlst%DT,  DTRT_TER
          write(6,*)  "reset DTRT_TER=nlst%DT "
          DTRT_TER=nlst%DT
    endif
    if(nlst%DT/DTRT_TER .ne. real(int(nlst%DT) / int(DTRT_TER)) ) then 
         write(6,*)  "nlst%DT,  DTRT_TER = ",nlst%DT,  DTRT_TER
         call hydro_stop("module_namelist: DT not a multiple of DTRT_TER")
    endif
    if(nlst%DT/DTRT_CH .ne. real(int(nlst%DT) / int(DTRT_CH)) ) then 
         write(6,*)  "nlst%DT,  DTRT_CH = ",nlst%DT,  DTRT_CH
         call hydro_stop("module_namelist: DT not a multiple of DTRT_CH")
    endif
#ifdef MPP_LAND
  endif
#endif
    
    if(outStatic .ne. 0) OVRTSWCRT = 1
    nlst%SUBRTSWCRT = SUBRTSWCRT
    nlst%OVRTSWCRT = OVRTSWCRT
    nlst%dxrt0 = dxrt
    nlst%AGGFACTRT = AGGFACTRT
    nlst%GWBASESWCRT = GWBASESWCRT
    nlst%GWSOILCPL= GWSOILCPL
    nlst%gwChanCondSw = gwChanCondSw
    nlst%gwChanCondConstIn = gwChanCondConstIn
    nlst%gwChanCondConstOut = gwChanCondConstOut
    nlst%gwIhShift = gwIhShift
    nlst%GwSpinCycles = GwSpinCycles
    nlst%GwPreCycles = GwPreCycles
    nlst%GwPreDiag = GwPreDiag
    nlst%GwSpinUp = GwSpinUp
    nlst%GwPreDiagInterval = GwPreDiagInterval
    nlst%TERADJ_SOLAR = TERADJ_SOLAR
    nlst%sys_cpl = sys_cpl
    nlst%rst_typ = rst_typ
    nlst%rst_bi_in = rst_bi_in
    nlst%rst_bi_out = rst_bi_out
    nlst%order_to_write = order_to_write
! files
    nlst%route_topo_f   =  route_topo_f
    nlst%route_chan_f = route_chan_f 
    nlst%route_link_f = route_link_f
    nlst%route_lake_f =route_lake_f
    nlst%route_direction_f =  route_direction_f
    nlst%route_order_f =  route_order_f
    nlst%gwbasmskfil =  gwbasmskfil
    nlst%gwstrmfil =  gwstrmfil
    nlst%geo_finegrid_flnm =  geo_finegrid_flnm
    nlst%udmap_file =  udmap_file
    nlst%UDMP_OPT = UDMP_OPT
    nlst%GWBUCKPARM_file =  GWBUCKPARM_file
    nlst%overbank = overbank
    nlst%bankWght = bankWght
    nlst%domain_msk_opt = domain_msk_opt
    nlst%dabe = dabe
    nlst%factorIn = factorIn
    nlst%baseflowFactor = baseflowFactor
  
    if(minDTCT0 .le. 0) minDTCT0 = 0.2
    nlst%minDTCT0 = minDTCT0

    if(minSlope .le. 0) minSlope = 0.002
    if(maxSlope .le. 0) maxSlope = 5    
    nlst%minSlope = minSlope
    nlst%maxSlope = maxSlope
    
#ifdef WRF_HYDRO_NUDGING
    nlst%nudgingParamFile      = nudgingParamFile
    nlst%netWkReExFile         = netWkReExFile
    nlst%readTimesliceParallel = readTimesliceParallel
    nlst%temporalPersistence   = temporalPersistence
    nlst%nudgingLastObsFile    = nudgingLastObsFile
#endif


   nlst%waveSwitch = 0

#ifdef MPP_LAND
  if(my_id .eq. IO_id) then
#endif
#ifdef HYDRO_D
     write(6,*) "output of the namelist file "
    write(6,*) "nlst%udmap_file ", trim(nlst%udmap_file)
    write(6,*) "nlst%UDMP_OPT ", nlst%UDMP_OPT
    write(6,*) " nlst%RT_OPTION ", RT_OPTION
    write(6,*) " nlst%CHANRTSWCRT ", CHANRTSWCRT
    write(6,*) " nlst%GW_RESTART  ", GW_RESTART 
    write(6,*) " nlst%RSTRT_SWC   ", RSTRT_SWC  
    write(6,*) " nlst%channel_option ", channel_option
    write(6,*) " nlst%DTRT_TER   ", DTRT_TER
    write(6,*) " nlst%DTRT_CH   ", DTRT_CH
    write(6,*) " nlst%SUBRTSWCRT ", SUBRTSWCRT
    write(6,*) " nlst%OVRTSWCRT ", OVRTSWCRT
    write(6,*) " nlst%dxrt0 ", dxrt
    write(6,*) " nlst%AGGFACTRT ", AGGFACTRT
    write(6,*) " nlst%GWBASESWCRT ", GWBASESWCRT
    write(6,*) " nlst%GWSOILCPL ", GWSOILCPL
    write(6,*) " nlst%gwChanCondSw ", gwChanCondSw
    write(6,*) " nlst%gwChanCondConstIn ", gwChanCondConstIn
    write(6,*) " nlst%gwChanCondConstOut ", gwChanCondConstOut
    write(6,*) " nlst%gwIhShift ", gwIhShift
    write(6,*) " nlst%GwSpinCycles ", GwSpinCycles
    write(6,*) " nlst%GwPreDiag ", GwPreDiag
    write(6,*) " nlst%GwPreDiagInterval ", GwPreDiagInterval
    write(6,*) " nlst%TERADJ_SOLAR ", TERADJ_SOLAR
    write(6,*) " nlst%sys_cpl ", sys_cpl
    write(6,*) " nlst%rst_typ ", rst_typ
    write(6,*) " nlst%order_to_write ", order_to_write
    write(6,*) " nlst%route_topo_f   ",  route_topo_f
    write(6,*) " nlst%route_chan_f ", route_chan_f 
    write(6,*) " nlst%route_link_f ", route_link_f
    write(6,*) " nlst%route_lake_f ",route_lake_f
    write(6,*) " nlst%route_direction_f ",  route_direction_f
    write(6,*) " nlst%route_order_f ",  route_order_f
    write(6,*) " nlst%gwbasmskfil ",  gwbasmskfil
    write(6,*) " nlst%gwstrmfil ",  gwstrmfil
    write(6,*) " nlst%geo_finegrid_flnm ",  geo_finegrid_flnm
#ifdef WRF_HYDRO_NUDGING
    write(6,*) " nlst%nudgingParamFile",       trim(nudgingParamFile)
    write(6,*) " nlst%netWkReExFile",          trim(netWkReExFile)
    write(6,*) " nlst%readTimesliceParallel",  readTimesliceParallel
    write(6,*) " nlst%temporalPersistence",    temporalPersistence
    write(6,*) " nlst%nudgingLastObsFile",     trim(nudgingLastObsFile)
#endif
#endif /* HYDRO_D */
#ifdef MPP_LAND
  endif
#endif


! derive rtFlag
      nlst%rtFlag = 1
      if(channel_option .eq. 4) nlst%rtFlag = 0
!      if(CHANRTSWCRT .eq. 0 .and.  SUBRTSWCRT .eq. 0 .and. OVRTSWCRT .eq. 0 .and. GWBASESWCRT .eq. 0) nlst%rtFlag = 0
      if(SUBRTSWCRT .eq. 0 .and. OVRTSWCRT .eq. 0 .and. GWBASESWCRT .eq. 0 .and. CHANRTSWCRT .eq. 0) nlst%rtFlag = 0

      if(nlst%CHANRTSWCRT .ne. 0 .and. (nlst%SUBRTSWCRT .eq. 0 .and. nlst%OVRTSWCRT .eq. 0) ) nlst%OVRTSWCRT  = 1
!!!!  switch back shortly. There is a bug in initialization. This is a temp solution. 

!!!! check the namelist 
!     write(6,*) "check namelist file .... "
!      if(my_id .eq. io_id) then
!             call getFileAttReal(trim(geo_static_flnm), "DX", dx1)
!             call getFileAttReal(trim(geo_finegrid_flnm), "DX", dx2)
!             write(6,*)  "dx1 = ", dx1, "dx2 = ", dx2
!             if(DXRT .ne. dx2 ) then 
!                write(6,*) "Serious Problem: dx from geo_finegrid_flnm does not match hydro.namelist (dxrt,dx2)", dxrt,dx2
!             endif
!             if(dx2 .gt. 0) then
!                  i = int(dx1/dx2)
!                  if(i .ne. AGGFACTRT) then
!                      write(6,*) "Serious Problem: AGGFACTRT from geo files does not match hydro.namelist ", i, AGGFACTRT
!                  endif
!             else
!                write(6,*) "Serious Problem: dx from geo_finegrid_flnm does not match hydro.namelist "
!             endif
!        endif

      write(6,*) "finish reading namelist ... "
      return
    end subroutine read_rt_nlst


end module module_namelist
