!23456789
       Program main 
        implicit none
        character(len=256) :: lakencFile
        logical :: exist
        integer :: nlakes, n
        real*8, allocatable, dimension(:) ::  WeirC,WeirL, WeirH, OrificeC, OrificeA, OrificeE, LkArea
        real, allocatable, dimension(:) :: alt, lat, lon
        integer, allocatable, dimension(:) :: lakeid
        integer iargc

        if(IARGC() .ne. 1) then
            print*, "Usage: lakenc2txt.exe LAKEPARM.nc"
            print*, "transfer the nc file into LAKEPARM.TBL"
            stop 999
        endif

        call getarg(1,lakencFile)

        call get_file_dimension0(trim(lakencFile), "nlakes", nlakes)
  
        allocate(LkArea(nlakes)) 
        allocate(WeirC(nlakes)) 
        allocate(WeirL(nlakes)) 
        allocate(WeirH(nlakes)) 
        allocate(OrificeC(nlakes)) 
        allocate(OrificeA(nlakes)) 
        allocate(OrificeE(nlakes)) 
        allocate(alt(nlakes)) 
        allocate(lat(nlakes)) 
        allocate(lon(nlakes)) 
        allocate(lakeid(nlakes)) 

        call get_1d_netcdf_int(trim(lakencFile), "lake_id", lakeid,nlakes) 
        call get_1d_netcdf_real8(trim(lakencFile), "LkArea", LkArea,nlakes) 
        call get_1d_netcdf_real8(trim(lakencFile), "LkMxH", WeirH,nlakes) 
        call get_1d_netcdf_real8(trim(lakencFile), "WeirC", WeirC,nlakes) 
        call get_1d_netcdf_real8(trim(lakencFile), "WeirL", WeirL,nlakes) 
        call get_1d_netcdf_real8(trim(lakencFile), "OrificeC", OrificeC,nlakes) 
        call get_1d_netcdf_real8(trim(lakencFile), "OrificeA", OrificeA,nlakes) 
        call get_1d_netcdf_real8(trim(lakencFile), "OrificeE", OrificeE,nlakes) 
        call get_1d_netcdf_real(trim(lakencFile), "lat", lat,nlakes) 
        call get_1d_netcdf_real(trim(lakencFile), "lon", lon,nlakes) 
        call get_1d_netcdf_real(trim(lakencFile), "alt", alt,nlakes) 

        open(12, file="LAKEPARM.TBL", form="formatted")
        write(12,*)  "lake    LkArea  LkMxH   WeirC   WeirL   WeirH   OrificeC        OrificeA        OrificeE        lat     long    elevation"
        do n = 1, nlakes
           write(12, 101) lakeid(n), LKArea(n), WeirH(n), WeirC(n),WeirL(n), &
                 WeirH(n)+1,  OrificeC(n),OrificeA(n), OrificeE(n), &
                 lat(n), lon(n), alt(n)
        enddo
101     format(I8,8f12.4,2f12.6, f12.3)
        close(12)

        stop 
        end

       subroutine get_1d_netcdf_real(inFile, varName, var,nsize)
          implicit none
#include <netcdf.inc>
            integer                            :: ncid !! the file identifier
            character(len=*)  :: varName, inFile
            real, dimension(nsize) :: var
            logical                            :: fatal_if_error
            integer :: varid, iret, nsize
            fatal_if_error = .true.
            iRet = nf_open(trim(inFile), nf_nowrite, ncid)
            iRet = nf_inq_varid(ncid, varName, varid)
            if (iret /= 0) then
               if (fatal_IF_ERROR) then
                  print*,  "Error:  get_1d_netcdf_real: variable: " // trim(varName)
                  stop
               end if
            end if
            iRet = nf_get_var_real(ncid, varid, var(1:nsize) )
            if (iRet /= 0) then
               print*, "Error: get_1d_netcdf_int: values: " // trim(varName)
               stop
            end if
            iret = nf_close(ncid)
       end subroutine get_1d_netcdf_real

       subroutine get_1d_netcdf_real8(inFile, varName, var,nsize)
          implicit none
#include <netcdf.inc>
            integer                            :: ncid !! the file identifier
            character(len=*)  :: varName, inFile
            real*8, dimension(nsize) :: var
            logical                            :: fatal_if_error
            integer :: varid, iret, nsize
            fatal_if_error = .true.
            iRet = nf_open(trim(inFile), nf_nowrite, ncid)
            iRet = nf_inq_varid(ncid, varName, varid)
            if (iret /= 0) then
               if (fatal_IF_ERROR) then
                  print*,  "Error:  get_1d_netcdf_real: variable: " // trim(varName)
                  stop
               end if
            end if
            iRet = nf_get_var_double(ncid, varid, var(1:nsize) )
            if (iRet /= 0) then
               print*, "Error: get_1d_netcdf_int: values: " // trim(varName)
               stop
            end if
            iret = nf_close(ncid)
       end subroutine get_1d_netcdf_real8

       subroutine get_1d_netcdf_int(inFile, varName, var,nsize)
          implicit none
#include <netcdf.inc>
            integer                            :: ncid !! the file identifier
            character(len=*)  :: varName, inFile
            integer, dimension(nsize)  :: var
            logical                            :: fatal_if_error
            integer :: varid, iret, nsize
            fatal_if_error = .true.
            iRet = nf_open(trim(inFile), nf_nowrite, ncid)
            iRet = nf_inq_varid(ncid, varName, varid)
            if (iret /= 0) then
               if (fatal_IF_ERROR) then
                  print*,  "Error:  get_1d_netcdf_real: variable: " // trim(varName)
                  stop
               end if
            end if
            iRet = nf_get_var_int(ncid, varid, var(1:nsize) )
            if (iRet /= 0) then
               print*, "Error: get_1d_netcdf_int: values: " // trim(varName)
               stop
            end if
            iret = nf_close(ncid)
       end subroutine get_1d_netcdf_int

      subroutine get_file_dimension0(fileName, var, size)
            implicit none
#include <netcdf.inc>
            character(len=*)::  fileName, var
            integer :: ncid , iret, size, dimid
            iret = nf_open(fileName, NF_NOWRITE, ncid)
            if (iret /= 0) then
               stop "In get_file_dimension() - Problem opening file"
            endif
            iret = nf_inq_dimid(ncid, trim(var), dimid)
            if (iret /= 0) then
               stop "In get_file_dimension() - nf_inq_dimid: "
            endif
            iret = nf_inq_dimlen(ncid, dimid, size)
            if (iret /= 0) then
               stop "In get_file_dimension() - nf_inq_dimlen:m"
            endif
            iret = nf_close(ncid)
      end subroutine get_file_dimension0



